//  Программа ДИСПЕНСЕР-2 V1.0 диспенсера паяльной пасты и флюса на 2 канала
//  alexeim2005"@mail.ru
//  Подключения:
//  дисплей 1106 или 1306 по I2C, SDA->A4, SCL->A5, меняем ниже в выборе модели дисплея
//  вход кнопки энкодера - D4
//  вход педали запуска дозы - D5
//  входа энкодера - D2, D3
//  выход сигнала  STEP - D12
//  выход сигнала DIR - D7
//  выход выбора драйвера EN 1  - D6
//  выход выбора драйвера EN 2 - D8
//  Управление кнопкой энкодера, кнопкой дозы и энкодером
//  Нажатия кнопки энкодера - одиночный клик -> выбор/возврат
//  Нажатия кнопки педали - одиночный клик -> запуск

#include <EEPROM.h>   // --- стандартная библиотека работы с eprom
#include "U8glib.h"   // --- модифицированная по шрифтам библиотека https://arduinoplus.ru/u8glib-oled-ekran-i-ssd1306/

// выбор модели дисплея
//U8GLIB_SH1106_128X64 u8g(U8G_I2C_OPT_NONE); // I2C / TWI
U8GLIB_SH1106_128X64 u8g(U8G_I2C_OPT_DEV_0 | U8G_I2C_OPT_FAST); // Dev 0, Fast I2C / TWI
//U8GLIB_SSD1306_128X64 u8g(U8G_I2C_OPT_NONE | U8G_I2C_OPT_DEV_0); // I2C / TWI
//U8GLIB_SSD1306_128X64 u8g(U8G_I2C_OPT_DEV_0 | U8G_I2C_OPT_NO_ACK | U8G_I2C_OPT_FAST); // Fast I2C / TWI

// определяем входа подключения кнопок
#define BTN1 4               // кнопка энкодера подключена к пин 4
#define BTN2 5               // кнопка педали дозы подключена к пин 5
#include "GyverButton.h"     // библиотека обработки кнопок https://github.com/GyverLibs/GyverButton
GButton butt1(BTN1);         //инициализация библиотеки для кнопки энкодера
GButton butt2(BTN2);         //инициализация библиотеки для кнопки педали

//определяем входа для подключения энкодера
#define ENC_A 2           // назначаем номер вывода энкодера
#define ENC_B 3           // назначаем номер вывода энкодера

// определяем тип энкодера
#define ENC_TYPE 1        // тип энкодера, 0-(высота корпуса 4.5мм) или 1 - (высота корпуса 6мм)

// определяем ножки для управления драйверами шаговых двигателей
#define STEP_PIN 12   // назначаем номер вывода сигнала STEP
#define DIR_PIN 7     // назначаем номер вывода сигнала DIR
#define ENN1_PIN 6    // назначаем номер вывода сигнала EN1
#define ENN2_PIN 8    // назначаем номер вывода сигнала EN2

// --- Шпаргалка адресов массива данных хранимых в eprom:
// ядресация ячеек: startAdr + i + p + ch + man, где
// startAdr - начальный адрес массива в eeprom
// i, принимает значения 0,1,2,3, где 0 - шаг вперед, 1 - скорость вперед, 2 - шаг назад, 3 - скорость назад
// p, принимает значения 0,4,8, сответствует П1,П2,П3
// ch, принимает значение 0,12, что соответствует ВЫХ1, ВЫХ2
// man, принимает значение 0,24, что сответствует АВТО, РУЧ
// startAdr - 4 --> memFreqSTEP      // частота сигнала STEP
// startAdr - 8 --> не используется  // не используется
// startAdr - 12 --> prescaler       // предделитель Таймера 1
// startAdr - 16 --> time_save_scr   // время срабатывания хранителя дисплея

//массив для сохраненых значений шагов и скоростей
const byte Arr_size = 4;    // задаем размер массива
byte setArray [Arr_size];   // обьявляем массив
//byte str_setArray = 0;     // переменная массива

// переменные для работы с eprom
unsigned int startAdr = 32 ;   // начальный адрес массива в eeprom от 32 до 900
int number_mem = 1;       // текущий номер установки памяти
int channel = 1;          // переменная номера выхода
int ch = 0;               // переменная приращения для расчета адреса ячейки при выборе выхода
int pos_set = 0;          // переменная приращения для расчета адреса ячейки при выборе шагов и скоростей
int p = 0;                // переменная приращения для расчета адреса ячейки памяти при выборе номера памяти
int man = 0;              // переменная приращения для расчета адреса ячейки памяти при выборе режима "РУЧ"

// переменные координат дисплея
int x = 0;                      // позиция X
int y = 0;                      // позиция Y

// переменные millis
unsigned long default_time_save_scr = 60000;    // по умолчанию временя срабатывания хранителя дисплея 1 мин = 60 сек*1000 мсек
unsigned long max_time_save_scr = 20 * default_time_save_scr;   // 20 мин = 20 * 1 мин
unsigned long time_save_scr = 0;                // ранее сохраненное времени срабатывания хранителя дисплея
unsigned long carrent_millis = 0;               // переменная текущего значения millis()
unsigned long old_millis = 0;                   // переменная прошлого значения millis()

// переменные
float Freq_CPU = 16000000;  // частота кварца процессора в Гц для расчетов
byte defaultPrescaler = 1;  // по умолчанию предделитель Таймера 1, может быть равен 1,8
byte prescaler = 1;         // текущий предделитель Таймера 1, может быть равен 1,8

// переменные для частоты генерации импульсов STEP
byte defaultMicroSTEP = 8;        //множитель микро шагов по умолчанию, равен количеству установленных микрошагов на драйвере TMC2208

byte defaultFactorSTEP = 1;       // множитель шагоd по умолчанию
byte maxFactorSTEP = 8;           // максимальное значение множителя микрошагов

float defaultFreqSTEP = 6000;     // ### частота по умолчанию в Гц сигнала STEP (c учетом микрошагов) = номинальная частота импульсов двигателя (в моем случае) 800Гц * множитель микро шагов по умолчанию
char correctionFreqSTEP = -6;     // ## переменная коррекции частоты сигнала STEP -128...+128
float memFreqSTEP = 100;          // считанная из eprom частота сигнала STEP
byte dividerStepForward = 255;    // ### экранный делитель шаг-вперед
byte dividerSpeedForward = 255;   // ### экранный делитель скорость-вперед
byte dividerStepBack = 255;       // ### экранный делитель шаг-назад
byte dividerSpeedBack = 255;      // делитель скорость-назад

float SetFreqSTEP = 0 ;           // переменная установленного на экране значения STEP c учетом микрошагов = номинальная частота * 8 микрошагов

float ImpSTEP = 0.00005;           // длительность импульса сигнала STEP в секундах
uint16_t OcrIMP = floor ((Freq_CPU / prescaler) / (1 / ImpSTEP) - 1);  // вычисляем делитель для ширины импульса STEP для записи в регистр сравнения B
uint16_t OcrSTEP;                  // переменная расчетного значения OcrSTEP
uint16_t minOcrSTEP;               // переменная расчетного значения FreqSTEP для записи в регистр сравнения А
uint16_t SetNumberSteps = 0;       // переменная заданного количества шагов двигателя
uint16_t CounterNumberSteps = 0;   // переменная счетчика количества шагов двигателя

//задаем строковый массив для меню настройки
const byte Arr_size_menu_strings = 6;    // задаем размер массива-количество строк в меню
const char * menu_strings[Arr_size_menu_strings] = {
  "Частота STEP",
  "Предделитель",
  "Погасить экран",
  "Сброс всех П1,П2,П3",
  "",
  "Выход"
};

// переменные энкодера и флаги энкодера

volatile int encCounter = 0;  // переменная значения encCounter, задавая ее здесь от 0 до 8 можно устанавливать начальное положение выделения на дисплее при включении
byte disp_encCounter = 0;     // переменная хранения значения дисплейного экодера
uint16_t old_encCounter = 0;  //переменная хранения прошлого значения энкодера
byte min_encCounter = 0;      // минимальное значение энкодера для ограничений ввода
uint16_t max_encCounter = 0;  // максимальное значение энкодера для ограничений ввода
byte set_encCounter = 1;      // переменная приращения энкодера

volatile boolean resetFlag;           // флаг сброса проверки кода энкодера
volatile byte curState, prevState;

// флаги

volatile boolean flag_press_key_enc = false; // флаг нажатия кнопки энкодера
volatile boolean flag_press_key_pedal = false; // флаг нажатия кнопки педали
volatile boolean flag_auto = true;           // флаг режимов работы "авто-руч"
volatile boolean flag_pos2 = false;          // флаг позиции рамки "вперед-шаг"
volatile boolean flag_pos3 = false;          // флаг позиции рамки "вперед-скорость"
volatile boolean flag_pos4 = false;          // флаг позиции рамки "назад-шаг"
volatile boolean flag_pos5 = false;          // флаг позиции рамки "назад скорость"
volatile boolean flag_pos6 = false;          // флаг позиции рамки "вперед"
volatile boolean flag_pos7 = false;          // флаг позиции рамки "назад"
volatile boolean flag_one = false;           // флаг одного срабатывания
volatile boolean flag_one_1 = false;         // флаг одного срабатывания
volatile boolean flagStopTimer1 = false;     // флаг окончания счета
volatile boolean flag_loop = false;          // флаг цикла для закольцовывания таймера 1
volatile boolean save_scr = false;           // флаг вызова функции хранителя экрана
volatile boolean flag_znak = false;          // флаг знака для индикации работы шагового двигателя



// =============================== УСТАНОВКИ ===============================

void setup() {

  //Serial.begin(9600);
  //Serial.println();

  // --- назначаем прерывания от вращения энкодера

  attachInterrupt(0, int0, CHANGE);
  attachInterrupt(1, int0, CHANGE);

  // --- установки для кнопки энкодера

  butt1.setDebounce(80);            // настройка антидребезга (по умолчанию 30 мс)
  butt1.setTimeout(300);            // настройка таймаута на удержание (по умолчанию 500 мс)
  butt1.setClickTimeout(350);       // настройка таймаута между кликами (по умолчанию 300 мс)
  butt1.setTickMode(AUTO);          // устанавливаем автоматичекий опрос функциией кнопки
  butt1.setType(HIGH_PULL);         // HIGH_PULL - кнопка подключена к GND, пин подтянут к VCC (BTN_PIN --- КНОПКА --- GND)
  butt1.setDirection(NORM_OPEN);    // NORM_OPEN - нормально-разомкнутая кнопка

  // --- установки для кнопки педали

  butt2.setDebounce(50);            // настройка антидребезга (по умолчанию 30 мс)
  butt2.setTimeout(100);            // настройка таймаута на удержание (по умолчанию 500 мс)
  butt2.setClickTimeout(300);       // настройка таймаута между кликами (по умолчанию 300 мс)
  butt2.setTickMode(AUTO);          // устанавливаем автоматический опрос кнопки педали
  butt2.setType(HIGH_PULL);         // HIGH_PULL - кнопка подключена к GND, пин подтянут к VCC (BTN_PIN --- КНОПКА --- GND)
  butt2.setDirection(NORM_OPEN);    // NORM_OPEN - нормально-разомкнутая кнопка

  pinMode (2, INPUT); // задаем режим работы входов энкодера
  pinMode (4, INPUT); // задаем режим работы

  // считываем в цикле в массив setArray из eprom ранее сохраненые значения

  for (int i = 0; i < Arr_size; i++) {
    setArray [i] = EEPROM.read (startAdr + i); // читаем из eprom и пишем в массив данные начиная с startAdr
  }


  // считываем  из eprom ранее сохраненые значениеmemFreqSTEP

  EEPROM.get (startAdr - 4, memFreqSTEP);                //читаем из eeprom
  if (memFreqSTEP < 1 || memFreqSTEP > defaultFreqSTEP) {
    EEPROM.put (startAdr - 4 , defaultFreqSTEP);          // пишется в eprom если вышли из диапазона
  }


  // считываем  из eprom ранее сохраненые значение Prescaler

  EEPROM.get (startAdr - 12, prescaler);                    //читаем из eeprom
  if (prescaler != defaultPrescaler || prescaler != 8) {
    EEPROM.put (startAdr - 12, defaultPrescaler);          // пишется в eprom один раз при первом запуске
  }


  // считываем  из eprom ранее сохраненые значение SaveScreen

  EEPROM.get (startAdr - 16, time_save_scr);                    //читаем из eeprom
  if (time_save_scr < 0 || time_save_scr > max_time_save_scr) {
    EEPROM.put (startAdr - 16, default_time_save_scr);           // пишется в eprom один раз при первом запуске
  }

  // настройки для выхода STEP
  pinMode(STEP_PIN, OUTPUT);     // назначаем STEP_PIN как выход
  digitalWrite(STEP_PIN, LOW);   // переводим в низкий уровень
  // настройки для выхода DIR
  pinMode(DIR_PIN, OUTPUT);     // назначаем DIR_PIN как выход
  digitalWrite(DIR_PIN, LOW);   // переводим в низкий уровень
  // настройки для выхода EN1
  pinMode(ENN1_PIN, OUTPUT);     // назначаем EN1_PIN как выход для выбора драйвера 1
  digitalWrite(ENN1_PIN, HIGH);  // отключаем драйвер 1
  // настройки для выхода EN2
  pinMode(ENN2_PIN, OUTPUT);     // назначаем EN1_PIN как выход для выбора драйвера 2
  digitalWrite(ENN2_PIN, HIGH);  // отключаем драйвер 2



  dispMiniature (); // выводим заставку
  dispMain();   // вызываем процедуру вывода основного экрана

}

// ============================== КОНЕЦ УСТАНОВОК ==============================



// [[[[[[[[[[[[[[[[[  Обработка прерывания от энкодера   ]]]]]]]]]]]]]]]]]
// алгоритм со сбросом от Ярослава Куруса https://vk.com/wall-97877471_572608, оптимизированный Alex Gyver https://alexgyver.ru/encoder/
void int0()
{
  curState = digitalRead(ENC_A) | digitalRead(ENC_B) << 1;
  if (resetFlag && curState == 0b11) {
    if (prevState == 0b10) (encCounter += set_encCounter);
    if (prevState == 0b01) (encCounter -= set_encCounter);
    resetFlag = 0;
  }
  if (curState == 0b00) resetFlag = 1;
  prevState = curState;
}

// =========== ПРОЦЕДУРА ВЫВОДА НА ДИСПЛЕЙ ОСНОВНОГО ЭКРАНА ===========

void dispMiniature () {
  // =========== РИСУЕМ заставку ============
  u8g.firstPage();
  do {
    u8g.setFont(rus10x20);  // устанавливаем шрифт
    u8g.setPrintPos(20, 25); u8g.print("ДОЗАТОР-2");     //x и y – координаты левого нижнего угола символа, выводим текст
    u8g.setPrintPos(40, 55); u8g.print("V 1.0");          //x и y – координаты левого нижнего угола символа, выводим текст
  } while ( u8g.nextPage() );
  // =========== закончили рисовать ===========
  delay (2000);
}

void dispMain () {
  u8g.firstPage();
  do {
    // =========== РИСУЕМ ОСНОВНОЙ ЭКРАН ============
    // рисуем таблицу

    u8g.drawFrame (0, 0, 127, 63);     //наружняя рамка (x и y – координаты левого верхнего угла,w – ширина (по х) прямоугольника,h – высота (по у) прямоугольника)
    u8g.drawVLine (32, 15, 47);        // первая вертикальная линия (x и y – координаты верхней точки линии,w – длина линии)
    u8g.drawVLine (63, 1, 61);         // вторая вертикальная линия (x и y – координаты верхней точки линии,w – длина линии)
    u8g.drawVLine (95, 1, 61);        // третья вертикальная линия (x и y – координаты верхней точки линии,w – длина линии)
    u8g.drawHLine(1, 14, 94);         // первая горизонтальная линия (x и y – координаты верхней точки линии,w – длина линии)
    u8g.drawHLine(1, 24, 126);         // вторая горизонтальная линия (x и y – координаты верхней точки линии,w – длина линии)
    u8g.drawHLine(1, 46, 126);         // третья горизонтальная линия (x и y – координаты верхней точки линии,w – длина линии)

    u8g.setFont(my5x7rus);  // устанавливаем шрифт

    u8g.setPrintPos(8, 21);  u8g.print("шаг");    //x и y – координаты левого нижнего угола символа, выводим текст
    u8g.setPrintPos(37, 21); u8g.print("скор");   //x и y – координаты левого нижнего угола символа, выводим текст
    u8g.setPrintPos(73, 21);  u8g.print("шаг");      //x и y – координаты левого нижнего угола символа, выводим текст
    u8g.setPrintPos(99, 10); u8g.print("МНОЖ");     //x и y – координаты левого нижнего угола символа, выводим текст
    u8g.setPrintPos(99, 21); u8g.print("шага");     //x и y – координаты левого нижнего угола символа, выводим текст

    //--- вывод "ВПЕРЕД"

    if (disp_encCounter == 6)  {
      u8g.drawFrame (1, 1, 62, 13);     // выводим прямоугольник
    }
    if (flag_pos6 == false) {
      u8g.setPrintPos(15, 10); u8g.print("ВПЕРЕД");  // выводим "ВПЕРЕД" (x и y – координаты левого нижнего угола символа)
    }
    else {
      u8g.setPrintPos(15, 10); u8g.print("ВПЕРЕД");  // выводим "ВПЕРЕД" (x и y – координаты левого нижнего угола символа)
      u8g.drawFrame (2, 2, 60, 11);
    }

    //--- вывод "НАЗАД"

    if (disp_encCounter == 7) {
      u8g.drawFrame (64, 1, 31, 13);      // выводим прямоугольник
    }
    if (flag_pos7 == false) {
      u8g.setPrintPos(71, 10); u8g.print("НАЗ");  // выводим "НАЗ" (x и y – координаты левого нижнего угола символа)
    }
    else {
      u8g.setPrintPos(71, 10); u8g.print("НАЗ");  // выводим "НАЗ" (x и y – координаты левого нижнего угола символа)
      u8g.drawFrame (65, 2, 29, 11);
    }

    u8g.setFont(u8g_font_9x15);  // устанавливаем шрифт

    // --- вывод на дисплей переменнной "ШАГ ВПЕРЕД"
    y = 40;
    if (setArray[0] <= 9) {
      x = 12;
    }
    else if (setArray[0] <= 99) {
      x = 7;
    }
    else  {
      x = 3;
    }

    if (disp_encCounter == 2)  {
      u8g.drawFrame (1, 25, 31, 21);
      if (flag_pos2 == true) {
        u8g.drawFrame (2, 26, 29, 19);
      }
    }
    if (flag_auto == false) {   // если флаг режима "АВТО" сброшен, то есть включен режим "РУЧ"
      u8g.setPrintPos(3, y);   //x и y – координаты левого нижнего угола символа
      u8g.print("<->");          // то выводим ХХ
    }
    else {                      // в противном случае у нас режим включен режим "АВТО"
      u8g.setPrintPos(x, y);   //x и y – координаты левого нижнего угола символа
      u8g.print(setArray[0]);   // то выводим содержимое члена массива
    }
    // --- прямой вывод на дисплей переменнной "СКОРОСТЬ ВПЕРЕД"

    if (setArray[1] <= 9) {
      x = 43;
    }
    else if (setArray[1] <= 99) {
      x = 39;
    }
    else {
      x = 34;
    }
    if (disp_encCounter == 3)  {
      u8g.drawFrame (33, 25, 30, 21);
      if (flag_pos3 == true) {
        u8g.drawFrame (34, 26, 28, 19);
      }
    }
    u8g.setPrintPos(x, y);  u8g.print(setArray[1]);//x и y – координаты левого нижнего угола символа, выводим переменную

    // --- вывод на дисплей переменнной "ШАГ НАЗАД"

    if (setArray[2] <= 9) {
      x = 75;
    }
    else if (setArray[2] <= 99) {
      x = 71;
    }
    else {
      x = 66;
    }
    if (disp_encCounter == 4)  {
      u8g.drawFrame (64, 25, 31, 21);
      if (flag_pos4 == true) {
        u8g.drawFrame (65, 26, 29, 19);
      }
    }
    u8g.setPrintPos(x, y); u8g.print(setArray[2]);//x и y – координаты левого нижнего угола символа, выводим переменную

    // --- вывод на дисплей переменнной "Множитель шага"

    if (setArray[3] <= 9) {
      x = 106;
    }
    else if (setArray[3] <= 99) {
      x = 102;
    }
    else {
      x = 98;
    }
    if (disp_encCounter == 5) {
      u8g.drawFrame (96, 25, 30, 21);
      if (flag_pos5 == true) {
        u8g.drawFrame (97, 26, 28, 19);
      }
    }
    u8g.setPrintPos(x, y); u8g.print(setArray[3]);// (x и y – координаты левого нижнего угола символа)

    u8g.setFont(rus6x10);  // устанавливаем шрифт

    // --- вывод НОМЕРА выхода

    if (disp_encCounter == 0) {
      u8g.drawFrame (64, 47, 31, 15);
      //u8g.drawFrame (65, 48, 29, 13);
    }
    u8g.setPrintPos(68, 58); u8g.print("ВЫХ");   // x и y – координаты левого нижнего угола символа, выводим текст
    u8g.setPrintPos(87, 58); u8g.print(channel);   // x и y – координаты левого нижнего угола символа, выводим текст

    // --- вывод режима "АВТО-РУЧ"

    if (disp_encCounter == 9) {
      u8g.drawFrame (33, 47, 30, 15);
      //u8g.drawFrame (34, 48, 28, 13);
    }
    if (flag_auto == true) {
      u8g.setPrintPos(37, 58);  u8g.print("АВТО");  // выводим "АВТО" (x и y – координаты левого нижнего угола символа)
    }
    else {
      u8g.setPrintPos(40, 58);  u8g.print("РУЧ");  // выводим "РУЧ" (x и y – координаты левого нижнего угола символа)
    }

    // --- вывод текущей ячейки памяти

    if (disp_encCounter == 1) {
      u8g.drawFrame (96, 47, 30, 15);
      //u8g.drawFrame (97, 48, 28, 13);
    }
    u8g.setPrintPos(104, 58);  u8g.print("П");  // выводим "П" (x и y – координаты левого нижнего угола символа)
    u8g.setPrintPos(113, 58);  u8g.print(number_mem); // выводим номер ячейки памяти (x и y – координаты левого нижнего угола символа)

    // --- вывод "УСТ"

    if (disp_encCounter == 8) {
      u8g.drawFrame (1, 47, 31, 15);
      // u8g.drawFrame (2, 48, 29, 13);
    }
    u8g.setPrintPos(8, 58);  u8g.print("УСТ");  // выводим "АВТО" (x и y – координаты левого нижнего угола символа)
    // =========== закончили рисовать ===========
  } while ( u8g.nextPage() );
}


// =========== ПРОЦЕДУРА ВЫВОДА НА ДИСПЛЕЙ МЕНЮ УСТАНОВОК ===========

void dispMenu() {

  u8g.firstPage();
  do {
    // =========== рисуем меню установок ===========
    u8g.setFont(rus6x10);   // устанавливаем шрифт
    x = 5;                  // смещение текста по оси Х
    for (int i = 0, y = -10; i < Arr_size_menu_strings; i++) {
      y = y + 10;
      if (i == encCounter)  {
        u8g.drawFrame (2,  y, 125, 11);           //рамка (x и y – координаты левого верхнего угла,w – ширина (по х) прямоугольника,h – высота (по у) прямоугольника)
        u8g.setPrintPos(x, 9 + y);                //x и y – координаты левого нижнего угола символа, выводим текст
        u8g.print(menu_strings[i]);               // печатаем
      }
      u8g.setPrintPos(x, 9 + y);               //x и y – координаты левого нижнего угола символа, выводим текст
      u8g.print(menu_strings[i]);
      if (y == 0) {
        u8g.setPrintPos(88, 9 + y);             //x и y – координаты левого нижнего угола символа, выводим текст
        u8g.print(round(memFreqSTEP));          // печатаем
        u8g.print("Гц");                        // печатаем
      }
      if (y == 10) {
        u8g.setPrintPos(105, 9 + y);            //x и y – координаты левого нижнего угола символа, выводим текст
        u8g.print("/"); u8g.print(prescaler);   // печатаем
      }
      if (y == 20) {
        u8g.setPrintPos(96, 9 + y);            //x и y – координаты левого нижнего угола символа, выводим текст
        u8g.print(time_save_scr / 60000);       // печатаем
        u8g.print("мин");                       // печатаем
      }
    }
    // =========== закончили рисовать ===========
  } while ( u8g.nextPage() );
}

// =========== ПРОЦЕДУРА ВЫВОДА НА ДИСПЛЕЙ УСТАНОВКИ ЧАСТОТЫ STEP ===========

void disSetStep() {
  u8g.firstPage();
  do {
    // ----- рисуем процедуру -----
    u8g.setFont(rus6x10);               // устанавливаем шрифт
    u8g.setPrintPos(1, 11);             //x и y – координаты левого нижнего угола символа, выводим текст
    u8g.print("Частота сигнала STEP");  // печатаем
    u8g.setPrintPos(1, 22);             //x и y – координаты левого нижнего угола символа, выводим текст
    u8g.print("по умолчанию  ");        // печатаем
    u8g.print(round(defaultFreqSTEP));  // печатаем
    u8g.print(" Гц");                   // печатаем
    u8g.setPrintPos(1, 33);             //x и y – координаты левого нижнего угола символа, выводим текст
    u8g.print("сохраненная   ");        // печатаем
    u8g.print(round(memFreqSTEP));      // печатаем
    u8g.print(" Гц");                   // печатаем
    u8g.setFont(rus10x20);              // устанавливаем шрифт
    u8g.setPrintPos(30, 56);            //x и y – координаты левого нижнего угола символа, выводим текст
    u8g.print(round (encCounter));      // печатаем
    u8g.setPrintPos(80, 56);            //x и y – координаты левого нижнего угола символа, выводим текст
    u8g.print("Гц");                    // печатаем

    // ----- закончили рисовать -----
  } while ( u8g.nextPage() );
}

// =========== ПРОЦЕДУРА ВЫВОДА НА ДИСПЛЕЙ УСТАНОВКИ ПРЕДДЕЛИТЕЛЯ ===========

void disPrescaler() {
  u8g.firstPage();
  do {
    // ----- рисуем процедуру -----
    u8g.setFont(rus6x10);               // устанавливаем шрифт
    u8g.setPrintPos(15, 11);            // x и y – координаты левого нижнего угола символа, выводим текст
    u8g.print("Предделитель");          // печатаем
    u8g.setPrintPos(15, 22);            // x и y – координаты левого нижнего угола символа, выводим текст
    u8g.print("по умолчанию  ");        // печатаем
    u8g.print(defaultPrescaler);        // печатаем
    u8g.setPrintPos(15, 33);            // x и y – координаты левого нижнего угола символа, выводим текст
    u8g.print("сохраненный   ");        // печатаем
    u8g.print(prescaler);               // печатаем
    u8g.setFont(rus10x20);              // устанавливаем шрифт
    u8g.setPrintPos(57, 56);            // x и y – координаты левого нижнего угола символа, выводим текст
    u8g.print(prescaler);               // печатаем
    // ----- закончили рисовать -----
  } while ( u8g.nextPage() );
}

// =========== ПРОЦЕДУРА ВЫВОДА НА ДИСПЛЕЙ УСТАНОВКИ ВРЕМЕНИ ХРАНИТЕЛЯ ЭКРАНА===========

void disSetSaveScreen() {
  u8g.firstPage();
  do {
    // ----- рисуем процедуру -----
    u8g.setFont(rus6x10);               // устанавливаем шрифт
    u8g.setPrintPos(1, 11);             //x и y – координаты левого нижнего угола символа, выводим текст
    u8g.print("Отключить экран");  // печатаем
    u8g.setPrintPos(1, 22);             //x и y – координаты левого нижнего угола символа, выводим текст
    u8g.print("по умолчанию   ");        // печатаем
    u8g.print(default_time_save_scr / 60000); // печатаем
    u8g.print(" мин");                   // печатаем
    u8g.setPrintPos(1, 33);             //x и y – координаты левого нижнего угола символа, выводим текст
    u8g.print("сохранено      ");        // печатаем
    u8g.print(time_save_scr / 60000);    // печатаем
    u8g.print(" мин");                   // печатаем
    u8g.setFont(rus10x20);              // устанавливаем шрифт
    u8g.setPrintPos(45, 56);            //x и y – координаты левого нижнего угола символа, выводим текст
    u8g.print(round (encCounter));      // печатаем
    u8g.setPrintPos(70, 56);            //x и y – координаты левого нижнего угола символа, выводим текст
    u8g.print("мин");                    // печатаем
    // ----- закончили рисовать -----
  } while ( u8g.nextPage() );
}

// =========== ПРОЦЕДУРА ВЫВОДА НА ДИСПЛЕЙ СБРОС П1,П2,П3  ===========

void dispResetMem() {
  u8g.firstPage();
  do {
    // ----- рисуем процедуру сброса -----
    u8g.setFont(rus6x10);    // устанавливаем шрифт
    u8g.setPrintPos(42, 10);      //x и y – координаты левого нижнего угола символа, выводим текст
    u8g.print("Сбросить");        // печатаем
    u8g.setPrintPos(20, 25);      //x и y – координаты левого нижнего угола символа, выводим текст
    u8g.print("все П1, П2, П3?"); // печатаем

    u8g.setFont(rus10x20);  // устанавливаем шрифт

    if (encCounter == 0) {
      u8g.drawBox (20, 38, 38, 20);   // закрашенный прямоугольник (x и y – координаты левого верхнего угла,w – ширина (по х) прямоугольника,h – высота (по у) прямоугольника)
      u8g.setColorIndex(0);           // меняем цвет пикселя 1 - горит, 0 - не горит
    }
    u8g.setPrintPos(24, 55);  u8g.print("НЕТ");  // выводим "НЕТ" (x и y – координаты левого нижнего угола символа)
    u8g.setColorIndex(1);             // меняем цвет пикселя 1 - горит, 0 - не горит
    if (encCounter == 1) {
      u8g.drawBox (70, 38, 38, 20);   // закрашенный прямоугольник (x и y – координаты левого верхнего угла,w – ширина (по х) прямоугольника,h – высота (по у) прямоугольника)
      u8g.setColorIndex(0);           // меняем цвет пикселя 1 - горит, 0 - не горит
    }
    u8g.setPrintPos(79, 55);  u8g.print("ДА");  // выводим "ДА" (x и y – координаты левого нижнего угола символа)
    u8g.setColorIndex(1);             // меняем цвет пикселя 1 - горит, 0 - не горит
    // ----- закончили рисовать -----
  } while ( u8g.nextPage() );
}


// =========== ПРОЦЕДУРА ОБРАБОТКИ ПОВОРОТА ЭНКОДЕРА и нажатия кнопки энкодера при установке шагов и скоростей ===========

void treatmentKeyEnc() {
  for (;;) {

    // задаем диапазон изменения энкодера
    if (encCounter <= min_encCounter) encCounter = min_encCounter;
    if (encCounter >= max_encCounter) encCounter = max_encCounter;
    setArray[pos_set] = encCounter;         // приравниваем элементу массива значение переменной
    if (old_encCounter != encCounter) {     // если прошлое значение не равно настоящему
      setArray[pos_set] = encCounter;       // приравниваем элементу массива значение переменной
      dispMain();                           // вызываем процедуру вывода на экран
      old_encCounter = setArray[pos_set];   // приравниваем прошлоe значение
    }
    if (butt1.isSingle ()) {              // проверяем на одиночное нажатие
      EEPROM.update (startAdr + pos_set + p + ch + man, setArray[pos_set]); // пишем в eprom установленное значение из массива
      butt1.resetStates();        // сброс всех флагов кнопки 1 в библиотеке
      break;  // прерываем цикл
    }
  }
}

// =========== НАСТРОЙКА ТАЙМЕРА 1 ===========

void startTimer1() {
  cli();                      // запрещаем все прерывания
  TCCR1A = 0;                 // сброс регистра А
  TCCR1B = 0;                 // сброс регистра В
  TCCR1B |= (1 << WGM12);     // режим счета импульсов, сброс при совпадении счетного регистра TCNT1 с регистром сравнения OCR1A
  OCR1A = OcrSTEP;            // пишем в регистр сравнения А "коэффициент деления" для периода STEP
  OCR1B = OcrIMP;             // пишем в регистр сравнения В ширину импульса для STEP
  TIFR1 = OCF1A;              // устанавливаем в регистре флагов прерывание по совпадению с регистром А
  TIFR1 = OCF1B;              // устанавливаем в регистре флагов прерывание по совпадению с регистром B
  TIMSK1 |= (1 << OCIE1A);    // разрешаем прерывание при совпадении с регистром А
  TIMSK1 |= (1 << OCIE1B);    // разрешаем прерывание при совпадении с регистром B
  if (prescaler == 1)   TCCR1B |= (1 << CS10);        // устанавливаем предделитель таймера = 1
  if (prescaler == 8) TCCR1B |= (1 << CS11);          // устанавливаем предделитель таймера = 8
  sei();                      // разрешаем все прерывания
  // ----------- конец настроек таймера -----------
}

// ----------- ПРОЦЕДУРА ВПЕРЕД-НАЗАД -----------

void ForvardBack() {
  flag_one_1 = false;             // флаг одного срабатывания сброшен
  for (;;) {

    // ------ проверяем условие в процедуре и если оно выполняется, то выводим на дисплей хранитель экрана
    scrSaver();                   // вызов функции хранителя экрана

    // формируем процедуру одиночного вывода на экран по состояною флага
    if (flag_one == false) {      // если флаг одиного срабатывания сброшен
      flag_one = true;            // то взводим флаг одного срабатывания
      if (disp_encCounter == 6) flag_pos6 = true; // если значение переменной, то взводим флаг вперед
      if (disp_encCounter == 7) flag_pos7 = true;    // если значение переменной, флаг взводим назад
      dispMain(); // вызываем процедуру вывода на экран
    }
    if  (butt2.isPress()) {                 // если педаль нажата
      if (channel == 1) {                   // если выбран выход 1
        digitalWrite(ENN2_PIN, HIGH);       // запрещаем работу драйвер 2
        digitalWrite(ENN1_PIN, LOW);        // выбираем драйвер 1
      }
      if (channel == 2) {                   // если выбран выход 1
        digitalWrite(ENN1_PIN, HIGH);       // запрещаем работу драйвер 1
        digitalWrite(ENN2_PIN, LOW);        // выбираем драйвер 2
      }
      if (disp_encCounter == 6) {                   // если положение выделения на шаг-вперед
        digitalWrite(DIR_PIN, HIGH);                // то устанавливаем направление вращения вперед - выдвигаем шток
      }
      if (disp_encCounter == 7) {                   // если положение выделения на скорость-вперед
        digitalWrite(DIR_PIN, LOW);                 // то устанавливаем направление вращения назад - задвигаем шток
      }
      if (flag_one_1 == false) {                      // если флаг одиного срабатывания сброшен
        flag_one_1 = true;                            // то взводим флаг одиного срабатывания
        OcrSTEP = floor (Freq_CPU / memFreqSTEP + correctionFreqSTEP - 1);  // вычисляем делитель для чаcтоты STEP
        flag_loop = true;                             // взводим флаг цикла для закольцовывания таймера
        startTimer1();                                // и запускаем таймер 1
      }
      butt2.resetStates();              // сброс всех флагов кнопки 2 в библиотеке
    }
    if  (butt2.isRelease()) {           // если педаль отпущена
      TCCR1B &= ~(1 << CS12 | 1 << CS11 | 1 << CS10); //останавливаем счет, устанавливаем предделитель в ноль
      digitalWrite(ENN2_PIN, HIGH);     // запрешаем работу драйвера 2
      digitalWrite(ENN1_PIN, HIGH);     // запрешаем работу драйвера 1
      flag_one_1 = false;               // сбрасываем флаг одного срабатывания 1
      butt2.resetStates();              // сброс всех флагов кнопки 2 в библиотеке
    }
    if (butt1.isSingle ()) {            // проверяем на одиночное нажатие
      if (disp_encCounter == 6) flag_pos6 = false;     // сбрасываем флаг "НАЗАД"
      if (disp_encCounter == 7) flag_pos7 = false;        // сбрасываем флаг "ВПЕРЕД"
      flag_loop = false;                // сбрасываем флаг цикла для закольцовывания таймера 1
      flag_one = false;                 // сбрасываем флаг одного срабатывания
      butt1.resetStates();              // сброс всех флагов кнопки 1 в библиотеке
      break;              // прерываем цикл
    }
  }
  old_encCounter = disp_encCounter;   // запоминаем прошлое значение
  if (disp_encCounter == 6) encCounter = 6;
  if (disp_encCounter == 7) encCounter = 7;
  // ----- конец вперед-назад -----
}

// ----------- ПРОЦЕДУРА ХРАНИТЕЛЯ ЭКРАНА -----------

void scrSaver() {
  carrent_millis = millis();            // присваиваем переменной текущено времени значение millis
  // "обнуляем" прошлое сохраненное время при условии,
  if (old_encCounter != encCounter || flag_press_key_enc == true || flag_press_key_pedal == true) { // если энкодер крутили, нажимали кнопку энкодера, кнопку педали, то
    old_millis = carrent_millis;        // "обнуляем" для дальнейшего сравнения прошлое значение переменной времени
  }
  // проверяем время включения хранителя экрана , при условии, что время не было установлено в 0
  if (time_save_scr > 0) {
    if (carrent_millis - old_millis >= time_save_scr && old_encCounter == encCounter && flag_press_key_enc == false && flag_press_key_pedal == false) {
      // если разница текущего времени и ранее запомненого больше заданного и энкодер не крутили и кнопки не нажимали, то
      old_millis = carrent_millis;        // запоминаем текущее значение времени
      old_encCounter = encCounter;        // запоминаем текущее значение энкодера
      randomSeed(analogRead(0));          // задаем функции начальное значение для расчета случайного числа
      for (;;) {
        x = random(0, 90);                // присваиваиваем перемнной х случайное значение в заданом диапазоне
        y = random(10, 64);               // присваиваиваем перемнной у случайное значение в заданом диапазоне
        // выводим на дисплей
        carrent_millis = millis();            // присваиваем переменной текущено времени значение millis
        if (carrent_millis - old_millis >= 1000) {
          // если разница текущего времени и ранее запомненого больше заданного и энкодер не крутили и кнопки не нажимали, то
          old_millis = carrent_millis;        // запоминаем текущее значение времени
          // вызываем процедуру вывода на экран
          u8g.firstPage();
          do {
            u8g.setFont(my5x7rus);                         // устанавливаем шрифт
            u8g.setPrintPos(x, y);  u8g.print("Спим?");    //x и y – координаты левого нижнего угола символа, выводим текст
          } while ( u8g.nextPage() );
        }
        if (old_encCounter != encCounter ||  butt1.isHolded() || butt2.isHolded()) { // если крутанули энкодер или нажали на кнопку энкодера или педали, то
          butt1.resetStates();        // сброс всех флагов кнопки 1 в библиотеке
          butt2.resetStates();        // сброс всех флагов кнопки 2 в библиотеке
          dispMain();                 // вызываем процедуру вывода на экран
          break;     // прерываем цикл
        }
      }
      encCounter = old_encCounter;            // восстанавливаем текущее значение энкодера
    }
  }
  flag_press_key_pedal = false; // сбрасываем флаг нажатия педали
  flag_one_1 = false;             // флаг одного срабатывания сброшен
}



// ========================================================
// ========================================================
// = =           О С Н О В Н О Й  Ц И К Л                ==
// ========================================================
// ========================================================



void loop() {

  // задаем диапазон изменения энкодера для основного экрана
  if (encCounter < 0) encCounter = 9;
  if (encCounter  > 9) encCounter = 0;
  disp_encCounter = encCounter;

  // ------ проверяем условие в процедуре и если оно выполняется, то выводим на дисплей хранитель экрана, если не выполняется, то возвращаемся назад
  scrSaver();               // вызов функции хранителя экрана

  // ----- вывод на дисплей только при повороте энкодера (изменении значения переменной)

  if (old_encCounter != disp_encCounter ) {   // если прошлое значение не равно настоящему
    dispMain();                               // то вызываем процедуру вывода на экран
    old_encCounter = disp_encCounter;         // и запоминаем прошлое значение
  }

  // ----- вывод на дисплей только по нажатию кнопки энкодера

  if ( flag_press_key_enc == true) {          // если флаг взведен (нажали на кнопку энкодера)
    for (int i = 0; i < Arr_size; i++) {
      setArray [i] = EEPROM.read (startAdr + i + p + ch + man); // читаем из eprom и пишем в массив данные
    }
    dispMain();                 //  вызываем процедуру вывода на экран
    flag_press_key_enc = false; // и сбрасываем флаг нажатия кнопки энкодера
  }

  // ----- ОПРОС ПЕДАЛИ И ЗАПУСК ДОЗЫ -----
  // ------ РЕЖИМ "АВТО" ------
  // ------ нажали на педаль дозы, автоматически даванули на установленное количество шагов, откатили поршень назад на установленное значение

  if (flag_auto == true) {              // если выбран режим АВТО
    if (butt2.isPress()) {              // если нажата педаль дозы
      flag_press_key_pedal = true;      // взводим флаг нажатия педали
      if (channel == 1) {               // если выбран выход 1
        digitalWrite(ENN2_PIN, HIGH);   // то запрешаем работу драйвера 2
        digitalWrite(ENN1_PIN, LOW);    // и выбираем драйвер 1
      }
      if (channel == 2) {               // если выбран выход 1
        digitalWrite(ENN1_PIN, HIGH);   // то запрешаем работу драйвера 1
        digitalWrite(ENN2_PIN, LOW);    // и выбираем драйвер 2
      }
      digitalWrite(DIR_PIN, HIGH);      // устанавливаем направление вращения - выдвигаем шток
      flag_one = false;                // сбрасываем флаг одиночного цикла
      for (;;) {
        if (flag_one == false) {       // если флаг сброшен
          flag_one = true;             // то взводим флаг
          SetNumberSteps = setArray[0] * defaultMicroSTEP * setArray[3];   // расчитываем количество шагов в зависимоти от установленного значения
          CounterNumberSteps = 0;                     // сбрасываем счетчик импульсов
          flagStopTimer1 = false;                     // флаг окончания счета
          OcrIMP = floor ((Freq_CPU / prescaler) / (1 / ImpSTEP) - 1);  // вычисляем делитель для ширины импульса STEP для записи в регистр сравнения B
          minOcrSTEP = floor (((Freq_CPU / prescaler) / memFreqSTEP + correctionFreqSTEP) - 1); // вычисляем минимальное значение счетчика для расчета делителя STEP
          OcrSTEP = minOcrSTEP + floor ((65535 - minOcrSTEP) / dividerSpeedForward  * (dividerSpeedForward - setArray[1]));  // вычисляем делитель для чаcтоты STEP
          flag_loop = false;                          // сбрасываем флаг зацикливания счета таймера
          startTimer1();                              // запускаем таймер 1
        }
        if (flagStopTimer1 == true) {   // если флаг окончания счета взведен
          break;                        // и прерываем цикл
        }
      }
      digitalWrite(DIR_PIN, LOW);            // устанавливаем направление вращения - задвигаем шток
      SetNumberSteps = setArray[2] * defaultMicroSTEP * setArray[3];         // расчитываем количество шагов в зависимоти от установленного значения
      CounterNumberSteps = 0;                           // сбрасываем счетчик импульсов
      flagStopTimer1 = false;                           // флаг окончания счета
      OcrSTEP = minOcrSTEP + floor ((65535 - minOcrSTEP) / dividerSpeedBack);  // вычисляем делитель для чаcтоты STEP
      startTimer1();                                    // запускаем таймер 1
      for (;;) {                            // крутим бесконечный цикл до тах пор пока счетчик не отсчитает и не взведет флаг
        if (flagStopTimer1 == true) {       // если флаг окончания счета таймера взведен
          digitalWrite(ENN1_PIN, HIGH);     // то запрещаем работу драйвера 1
          digitalWrite(ENN2_PIN, HIGH);     // то запрещаем работу драйвера 2
          flag_one = false;                // сбрасываем флаг одиночного цикла
          break; // прерываем цикл
        }
      }
      butt2.resetStates();        // сброс всех флагов кнопки 2 в библиотеке
    }
  }

  // ------ РЕЖИМ "РУЧ" ------
  // пока держим кнопку дозы давим постоянно, как отпустили откатили поршень назад на установленное значение

  if (flag_auto == false) {                 // если выбран режим РУЧ
    if  (butt2.isHolded()) {                // проводим проверку на удержание
      flag_press_key_pedal = true;          // взводим флаг нажатия педали
      if (channel == 1) {                   // если выбран выход 1
        digitalWrite(ENN2_PIN, HIGH);       // запрещаем работу драйвер 2
        digitalWrite(ENN1_PIN, LOW);        // выбираем драйвер 1
      }
      if (channel == 2) {                   // если выбран выход 1
        digitalWrite(ENN1_PIN, HIGH);       // запрещаем работу драйвер 1
        digitalWrite(ENN2_PIN, LOW);        // выбираем драйвер 2
      }
      digitalWrite(DIR_PIN, HIGH);                  // то устанавливаем направление вращения вперед - выдвигаем шток
      OcrIMP = floor ((Freq_CPU / prescaler) / (1 / ImpSTEP) - 1);  // вычисляем делитель для ширины импульса STEP для записи в регистр сравнения B
      minOcrSTEP = floor (((Freq_CPU / prescaler) / memFreqSTEP + correctionFreqSTEP) - 1); // вычисляем минимальное значение счетчика для расчета делителя STEP
      OcrSTEP = minOcrSTEP + floor ((65535 - minOcrSTEP) / dividerSpeedForward  * (dividerSpeedForward - setArray[1]));  // вычисляем делитель для чаcтоты STEP
      flag_loop = true;                             // взводим флаг цикла для зацикливания таймера
      startTimer1();                                // то запускаем таймер 1
      butt2.resetStates();        // сброс всех флагов кнопки 2 в библиотеке
    }
    if  (butt2.isRelease()) {     // если педаль отпущена
      TCCR1B &= ~(1 << CS12 | 1 << CS11 | 1 << CS10);                 //останавливаем счет, устанавливаем предделитель в ноль
      flag_loop = false;                                              // сбрасываем флаг цикла для закольцовывания таймера 1
      digitalWrite(DIR_PIN, LOW);                                     // устанавливаем направление вращения - задвигаем шток
      SetNumberSteps = setArray[2] * defaultMicroSTEP * setArray[3];  // расчитываем количество шагов в зависимоти от установленного значения, хрен его знает стоит ли окатывать с учетом множителя
      CounterNumberSteps = 0;                                         // сбрасываем счетчик импульсов
      flagStopTimer1 = false;                                         // сбрасываем флаг окончания счета
      OcrSTEP = minOcrSTEP + floor ((65535 - minOcrSTEP) / dividerSpeedBack);  // вычисляем делитель для чаcтоты STEP
      startTimer1();                                  // и запускаем таймер 1
      for (;;) {                            // крутим цикл до тех пор пока таймер 1 не отсчитает до конца и взведет флаг
        if (flagStopTimer1 == true) {       // если флаг окончания счета таймера 1 взведен
          digitalWrite(ENN1_PIN, HIGH);     // то запрешаем работу драйвера 1
          digitalWrite(ENN2_PIN, HIGH);     // то запрешаем работу драйвера 2
          break;  // и прерываем цикл
        }
      }
      butt2.resetStates();        // сброс всех флагов кнопки 2 в библиотеке
    }

  }
  // ----- окончание опроса педали и запуска дозы режим АВТО и РУЧ -----


  // <<<<<  ПРОВЕРЯЕМ НА ОДИН КЛИК В ОСНОВНОМ ЭКРАНЕ >>>>>

  if (butt1.isSingle())
  {

    // ----- ВЫБОР ВЫХОДА -----
    // выбор номера выхода по одному нажатию кнопки энкодера
    if (disp_encCounter == 0) {
      if (channel == 1) {
        channel = 2;
        ch = 12;
      }
      else {
        channel = 1;
        ch = 0;
      }
      for (int i = 0; i < Arr_size; i++) {
        setArray [i] = EEPROM.read (startAdr + i + p + ch + man); // читаем из eprom и пишем в массив данные
      }
      flag_press_key_enc = true; // взводим флаг срабатывания кнопки энкодера
      encCounter = 0;
    }



    // ----- ВЫБОР ЯЧЕЙКИ ПАМЯТИ П1,П2,П3 -----

    if (disp_encCounter == 1) {
      // задаем диапазон изменения количество ячеек памяти
      if (number_mem <= 1) number_mem = 1;
      if (number_mem >= 3) number_mem = 0;
      number_mem = number_mem + 1;
      // задаем приращение для расчета адреса
      if (number_mem == 1) p = 0;
      if (number_mem == 2) p = 4;
      if (number_mem == 3) p = 8;
      for (int i = 0; i < Arr_size; i++) {
        setArray [i] = EEPROM.read (startAdr + i + p + ch + man); // читаем из eprom и пишем в массив данные
      }
      dispMain(); // вызываем процедуру вывода на экран


    }

    // ----- ИЗМЕНЕНИЕ ПЕРЕМЕННОЙ "ВПЕРЕД ШАГ" -----

    if (disp_encCounter == 2) {               // выбор номера выхода по одному нажатию кнопки энкодера
      flag_pos2 = true;
      dispMain(); // вызываем процедуру вывода на экран
      pos_set = 0;
      encCounter = setArray[pos_set];         // присваиваем энкодеру значение из ячейки памяти
      min_encCounter = 1;                     // минимальное значение энкодера для ограничений ввода
      max_encCounter = dividerStepForward;    // максимальное значение энкодера для ограничений ввода
      treatmentKeyEnc();                      // вызов обработки поворота энкодера, нажатия кнопки, вывода на дисплей
      old_encCounter = disp_encCounter;       // запоминаем прошлое значение
      encCounter = 2;
      flag_pos2 = false;
      dispMain(); // вызываем процедуру вывода на экран
    }
    // ----- изменение переменной "ВПЕРЕД СКОРОСТЬ" -----

    if (disp_encCounter == 3) {             // выбор номера выхода по одному нажатию кнопки энкодера
      flag_pos3 = true;
      dispMain(); // вызываем процедуру вывода на экран
      pos_set = 1;
      encCounter = setArray[pos_set];       // присваиваем энкодеру значение из ячейки памяти
      min_encCounter = 1;                   // минимальное значение энкодера для ограничений ввода
      max_encCounter = dividerSpeedForward; // максимальное значение энкодера для ограничений ввода
      treatmentKeyEnc();                    // вызов обработки поворота энкодера, нажатия кнопки, вывода на дисплей
      old_encCounter = disp_encCounter;     // запоминаем прошлое значение
      encCounter = 3;
      flag_pos3 = false;
      dispMain(); // вызываем процедуру вывода на экран
    }
    // ----- изменение переменной "НАЗАД ШАГ" -----
    if (disp_encCounter == 4) {           // выбор номера выхода по одному нажатию кнопки энкодера
      flag_pos4 = true;
      dispMain(); // вызываем процедуру вывода на экран
      pos_set = 2;
      encCounter = setArray[pos_set];     // присваиваем энкодеру значение из ячейки памяти
      min_encCounter = 1;                 // минимальное значение энкодера для ограничений ввода
      max_encCounter = dividerStepBack;   // максимальное значение энкодера для ограничений ввода
      treatmentKeyEnc();                  // вызов обработки поворота энкодера, нажатия кнопки, вывода на дисплей
      old_encCounter = disp_encCounter;   // запоминаем прошлое значение
      encCounter = 4;
      flag_pos4 = false;
      dispMain(); // вызываем процедуру вывода на экран
    }
    // ----- изменение переменной "МНОЖИТЕЛЬ шагов" -----
    if (disp_encCounter == 5) {           // выбор номера выхода по одному нажатию кнопки энкодера
      flag_pos5 = true;
      dispMain(); // вызываем процедуру вывода на экран
      pos_set = 3;
      encCounter = setArray[pos_set];       // присваиваем энкодеру значение из ячейки памяти
      min_encCounter = defaultFactorSTEP;   // минимальное значение энкодера для ограничений ввода
      max_encCounter = maxFactorSTEP;       // максимальное значение энкодера для ограничений ввода
      treatmentKeyEnc();                    // вызов обработки поворота энкодера, нажатия кнопки, вывода на дисплей
      old_encCounter = disp_encCounter;     // запоминаем прошлое значение
      encCounter = 5;
      flag_pos5 = false;
      dispMain(); // вызываем процедуру вывода на экран
    }

    // ----- режим "ВПЕРЕД" -----

    if (disp_encCounter == 6) { // если выборан режим ВПЕРЕД
      ForvardBack();            // вызываем процедуру обработки
      flag_press_key_enc = true;  // взводим флаг срабатывания кнопки энкодера
    }

    // ----- РЕЖИМ "НАЗАД" -----
    if (disp_encCounter == 7) { // если выбран режимм НАЗАД
      ForvardBack();            // вызов обработки
      flag_press_key_enc = true;  // взводим флаг срабатывания кнопки энкодера
    }



    // ======================= РЕЖИМ УСТановки ===========================

    if (disp_encCounter == 8) {   // выбор номера выхода по одному нажатию кнопки энкодера
      encCounter = 0;             // переменная равно нулю

      for (;;) {                  // запускаем цикл
        // задаем диапазон изменения энкодера
        if (encCounter < 0) {
          //encCounter = 0;
          encCounter = Arr_size_menu_strings - 1;
        }
        if (encCounter > Arr_size_menu_strings - 1) {
          encCounter = 0;
          //encCounter = Arr_size_menu_strings - 1;
        }
        // ----- вывод на дисплей только при повороте энкодера
        if (old_encCounter != encCounter ) {  // если прошлое значение не равно настоящему
          dispMenu();                         // вызываем процедуру вывода на экран
          old_encCounter = encCounter;        // запоминаем прошлое значение
        }
        // ----- вывод на дисплей только по нажатию кнопки энкодера
        if ( flag_press_key_enc == true) {          // если флаг взведен (нажали на кнопку энкодера)
          dispMenu();                         // вызываем процедуру вывода на экран
          flag_press_key_enc = false; // и сбрасываем флаг нажатия кнопки энкодера
        }

        if (butt1.isSingle ()) {    // проверяем на одиночное нажатие

          // -------------------- МЕНЮ УСТАНОВОК НАЧАЛО --------------------

          // ------- Исполнение пункта меню "Частота STEP" ----------------

          if (encCounter == 0) {
            encCounter = memFreqSTEP;
            set_encCounter = 100;         // устанавливаем приращение счета энкодера
            disSetStep();                 // вызываем процедуру вывода на экран
            flag_press_key_enc == false;
            for (;;) {
              // задаем диапазон изменения энкодера
              if (encCounter <= 100) encCounter = 100;
              if (encCounter  >= defaultFreqSTEP) encCounter = defaultFreqSTEP;
              // ----- вывод на дисплей только при повороте энкодера
              if (old_encCounter != encCounter) {   // если прошлое значение не равно настоящему
                disSetStep();                       // вызываем процедуру вывода на экран
                old_encCounter = encCounter;        // запоминаем прошлое значение
              }
              if (butt1.isDouble ()) {                        // проверяем на двойное нажатие
                if (flag_press_key_enc == true) {
                  set_encCounter = 1;                         // устанавливаем приращение счета энкодера
                }
                else {
                  set_encCounter = 100;                       // устанавливаем приращение счета энкодера
                }
                flag_press_key_enc = ! flag_press_key_enc;    // инвертируем флаг
                butt1.resetStates();                          // сброс всех флагов кнопки 1 в библиотеке
              }

              if (butt1.isSingle ()) {                    // проверяем на одиночное нажатие
                memFreqSTEP = encCounter;
                EEPROM.put (startAdr - 4, memFreqSTEP);   //пишем в eeprom
                set_encCounter = 1;                       // устанавливаем приращение счета энкодера
                encCounter = 0;
                flag_press_key_enc = true;                // взводим флаг срабатывания кнопки энкодера
                break;  // прерываем цикл
              }
            }
          }



          // ------------ Исполнение пункта меню "Предделитель таймера" ----------------

          if (encCounter == 1) {
            encCounter = prescaler;
            disPrescaler();            // вызываем процедуру вывода на экран
            set_encCounter = 8;        // устанавливаем приращение счета энкодера
            encCounter = prescaler;
            old_encCounter = encCounter;
            for (;;) {
              // задаем диапазон изменения энкодера
              if (encCounter <= 1) encCounter = 1;
              if (encCounter  >= 8) encCounter = 8;
              // ----- вывод на дисплей только при повороте энкодера
              if (old_encCounter != encCounter ) {      // если прошлое значение не равно настоящему
                if (encCounter == 1) prescaler = 1;
                if (encCounter == 8) prescaler = 8;
                if (old_encCounter != encCounter) {      // если прошлое значение не равно настоящему
                  disPrescaler();                        // вызываем процедуру вывода на экран
                  old_encCounter = encCounter;           // запоминаем прошлое значение
                }
              }

              //Serial.println(encCounter);

              if (butt1.isSingle ()) {                  // проверяем на одиночное нажатие
                EEPROM.put (startAdr - 12, prescaler); //пишем в eeprom
                set_encCounter = 1;                     // устанавливаем приращение счета энкодера
                encCounter = 1;
                flag_press_key_enc = true;  // взводим флаг срабатывания кнопки энкодера
                break;    // прерываем цикл
              }
            }
          }

          // ------------ Исполнение пункта меню ПОГАСИТЬ ЭКРАН ------------

          if (encCounter == 2) {
            encCounter = time_save_scr / 60000;
            old_encCounter = 100;
            set_encCounter = 1;         // устанавливаем приращение счета энкодера
            for (;;) {
              // задаем диапазон изменения энкодера
              if (encCounter <= 0) encCounter = 0;
              if (encCounter  >= max_time_save_scr / 60000) encCounter = max_time_save_scr / 60000;
              // ----- вывод на дисплей только при повороте энкодера
              if (old_encCounter != encCounter ) {  // если прошлое значение не равно настоящему
                disSetSaveScreen();                 // вызываем процедуру вывода на экран
                old_encCounter = encCounter;        // запоминаем прошлое значение
              }
              if (butt1.isSingle ()) {              // проверяем на одиночное нажатие
                time_save_scr = encCounter * 60000;
                EEPROM.put (startAdr - 16, time_save_scr); //пишем в eeprom
                set_encCounter = 1;       // устанавливаем приращение счета энкодера
                old_encCounter = 100;
                encCounter = 2;
                flag_press_key_enc = true;  // взводим флаг срабатывания кнопки энкодера
                break;    // прерываем цикл
              }
            }
          }

          // -------------- Исполнение пункта меню "Сброс всех П1,П2,П3" ----------------

          if (encCounter == 3) {
            encCounter = 0;
            set_encCounter = 1;       // устанавливаем приращение счета энкодера
            for (;;) {
              // задаем диапазон изменения энкодера
              if (encCounter <= 0) encCounter = 0;
              if (encCounter  >= 1) encCounter = 1;
              // ----- вывод на дисплей только при повороте энкодера
              if (old_encCounter != encCounter ) {  // если прошлое значение не равно настоящему
                dispResetMem();                     // вызываем процедуру вывода на экран
                old_encCounter = encCounter;        // запоминаем прошлое значение
              }

              if (butt1.isSingle ()) {    // проверяем на одиночное нажатие
                if (encCounter == 0) {
                  encCounter = 3;
                  butt1.resetStates();        // сброс всех флагов кнопки 1 в библиотеке
                  break;   // прерываем цикл
                }

                // прописываем в весь диапазон сохранения данных
                if (encCounter == 1) {

                  for (int i = 0; i < Arr_size * 12; i = i + 4) {
                    for (int j = 0; j < Arr_size; j++) {
                      if (j == 3) {
                        EEPROM.update (startAdr + i + j, defaultFactorSTEP);  // прописываем defaultFactorSTEP
                      }
                      else {
                        EEPROM.update (startAdr + i + j, 255);    // прописываем 255 в остальные ячейки
                      }
                    }
                  }

                  // обновляем данные в текущий массив
                  for (int i = 0; i < Arr_size; i++) {
                    setArray [i] = EEPROM.read (startAdr + i + p + ch + man); // читаем из eprom и пишем в массив данные
                    Serial.println(setArray [i]);

                  }
                  set_encCounter = 1;         // устанавливаем приращение счета энкодера
                  encCounter = 3;
                  butt1.resetStates();        // сброс всех флагов кнопки 1 в библиотеке
                  flag_press_key_enc = true;  // взводим флаг срабатывания кнопки энкодера
                  break;  // прерываем цикл
                }
              }
            }
          }


          // ------------- Исполнение пункта меню "   " ----------------

          if (encCounter == 4) {

          }

          // ---------------- пункт меню "Выход" ----------------

          if (encCounter == 5) {
            encCounter = 1;
            butt1.resetStates();        // сброс всех флагов кнопки 1 в библиотеке
            break;  // прерываем цикл
          }
        }
      }
      old_encCounter = 0; // сбрасываем значение для запуска вывода на экран
      encCounter = 8;     // устанавливаем значение энкодера для вывода рамки на "УСТ"
    }
    // -------------------- КОНЕЦ МЕНЮ УСТАНОВКИ --------------------

    // ----- АВТО-РУЧНОЙ -----

    if (disp_encCounter == 9) {   // выбор по одному нажатию кнопки энкодера

      flag_auto = !flag_auto;     // инвертируем флаг
      if (flag_auto == true) {    // если флаг взведен
        man = 0;
      }
      else {                      // в противном случае если флаг сброшен
        man = 24;
      }
      encCounter = 9;             // устанавливаем значение энкодера для вывода рамки на "АВТО-РУЧ"
      flag_press_key_enc = true;  // взводим флаг срабатывания кнопки энкодера
    }
    butt1.resetStates();        // сброс всех флагов кнопки 1 в библиотеке
    butt2.resetStates();        // сброс всех флагов кнопки 2 в библиотеке
  }
}




// ++++++++++++++ Обработка прерывания при совпадении счетного регистра таймера1 с регистром сравнения А ++++++++++++++

ISR(TIMER1_COMPA_vect) {
  PORTB |= (1 << 4); // пишем 1 в 4 бит порта "В" (D12)
}

// ++++++++++++++ Обработка прерывания при совпадении счетного регистра таймера1 с регистром сравнения В ++++++++++++++

ISR(TIMER1_COMPB_vect) {
  PORTB &= ~(1 << 4);    // пишем  0 в 4 бит порта "B" (D12)
  CounterNumberSteps = CounterNumberSteps + 1;  // счетчик импульсов
  if (flag_loop == true) {       // если флаг цикла для закольцовывания таймера взведен то генерим по кругу до сброса flag_loop
  }
  else {                         // в противном случае (если флаг закольцовывания сброшен)
    if (CounterNumberSteps >= SetNumberSteps) {       // если счетчик импульсов больше или равен установленому количеству импульсов
      TCCR1B &= ~(1 << CS12 | 1 << CS11 | 1 << CS10); // то останавливаем счет устанавливая все предделители в ноль
      flagStopTimer1 = true;                          // и взводим флаг окончания счета
    }
  }
}
